unit ControlIndex;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  BDE,Dialogs,Db,DbTables;

type
  TReIndexEvent   = procedure(Sender : TObject;Var Table :TTable;Var State : TDataSetState) Of Object;
  TReIndexErrorEvent = procedure (Sender : TObject) of Object;

  TControlIndexs = class(TComponent)
  private
    fDatabaseName,
    fTableName : String;
    fActive    : Boolean;
    fShowMessageWhenFinish : Boolean;
    fOnBeforeReIndex,
    fOnAfterReIndex  : TReIndexEvent;
    fOnReIndexError  : TReIndexErrorEvent;

  protected
    procedure SetDatabaseName(Value : String);
    procedure SetTableName(Value    : String);
    procedure SetActive(Value : Boolean);
    procedure SetShowMessageWhenFinish(Value : Boolean);

  public
    Constructor Create(AComponent : TComponent);Override;
    Destructor  Destroy;Override;
    function  ReIndexs(DatabaseName,TableName : String;Var Error_Message : String) : Boolean;
    Procedure ReSecondIndex(Table: TTable; IndexName, TagName: String; IndexNum: Word);
  published
    property DatabaseName : String  read fDatabaseName write SetDatabaseName;
    property TableName    : String  read fTableName    write SetTableName;
    property ShowMessageWhenFinish : Boolean read fShowMessageWhenFinish write SetShowMessageWhenFinish;
    property Active       : Boolean read fActive       write SetActive;
    (* >> Assign Event << *)
    property OnReIndexError  : TReIndexErrorEvent read fOnReIndexError  write fOnReIndexError;
    property OnBeforeReIndex : TReIndexEvent      read fOnBeforeReIndex write fOnBeforeReIndex;
    property OnAfterReIndex  : TReIndexEvent      read fOnAfterReIndex  write fOnAfterReIndex;
  end;

procedure Register;

implementation

procedure TControlIndexs.SetShowMessageWhenFinish(Value : Boolean);
begin
     fShowMessageWhenFinish := Value;
end;

procedure TControlIndexs.SetDatabaseName(Value : String);
begin
     fDatabaseName := Value;
end;

procedure TControlIndexs.SetTableName(Value    : String);
begin
     fTableName := Value;
end;

procedure TControlIndexs.SetActive(Value : Boolean);
var fError_Message : String;
begin
     fActive := Value;
     If fActive  Then
        If ReIndexs(fDatabaseName,fTableName,fError_Message) = False Then
           If fError_Message <> 'OK..' Then
              MessageDlg(fError_Message,mtError,[mbOk],0);
end;

function TControlIndexs.ReIndexs(DatabaseName,TableName : String;Var Error_Message : String) : Boolean;
Var BDEResult : Word;
    CurrentTable : TTable;
    Sender       : TObject;
    CurrentTable_State : TDatasetState;
    Ignore       : Boolean;      
begin
    Result := False;
    Error_Message := '';

    CurrentTable := TTable.Create(Nil);
    CurrentTable.DatabaseName := DataBaseName;
    CurrentTable.TableName    := TableName;
    CurrentTable.Exclusive    := True;

    Screen.Cursor    := crSQLWait;

    Try
       CurrentTable.Close;
       CurrentTable.Exclusive    := True;
       CurrentTable.Open;
       Try
          // Event
          If Assigned(fOnBeforeReIndex) Then fOnBeforeReIndex(Sender,CurrentTable,CurrentTable_State);

          //Check(DBIRegenIndexes(CurrentTable.Handle));
          BDEResult := DBIRegenIndexes(CurrentTable.Handle);
          Case BDEResult Of
              DBIErr_NONE          : begin Error_Message := 'OK..'; Result := True; end;
              DBIErr_INVALIDHNDL   : Error_Message := 'The specified cursor handle is invalid or NULL. !!';
              DBIErr_NEEDEXCLACCESS: Error_Message := 'The table associated with Handle is opened in open shared mode. !!';
              DBIERR_NOTSUPPORTED  : Error_Message := 'SQL indexes cannot be regenerated.';
          Else
              If Assigned(fOnReIndexError) Then fOnReIndexError(Sender);
              Error_Message := 'Unexpected Error Return By BDE !!';
          end;
       Finally
          Screen.Cursor := crDefault;
          CurrentTable.Close;
          CurrentTable.Exclusive    := True;
          CurrentTable.Free;
          // Event
          If Assigned(fOnAfterReIndex) Then fOnAfterReIndex(Sender,CurrentTable,CurrentTable_State);

          If fShowMessageWhenFinish Then
             MessageDlg('Re-Index Process OK !!',mtInformation,[mbOk],0);
       end;
    Except
       MessageDlg('Table is Busy !! Cannot Open Table '+TableName+#13#10+
                  'In DatabaseName '+DatabaseName,mtError,[mbOk],0);

       Screen.Cursor := crDefault;
       CurrentTable.Close;
       CurrentTable.Exclusive    := True;
       CurrentTable.Free;
    end;
end;

Procedure TControlIndexs.ReSecondIndex(Table: TTable; IndexName, TagName: String; IndexNum: Word);
begin
  Check(DbiRegenIndex(Table.DBHandle, nil, PChar(Table.TableName), nil,
         PChar(IndexName), PChar(TagName), IndexNum));
end;

constructor TControlIndexs.Create(AComponent : TComponent);
begin
    Inherited Create(AComponent);
end;

Destructor  TControlIndexs.Destroy;
begin
     Inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('ThaiDelphiComponent', [TControlIndexs]);
end;

end.
